@@ -48,10 +48,8 @@ typedef struct {
 
 GType remote_viewer_get_type (void);
 
-RemoteViewer* remote_viewer_new(const gchar *uri,
-                                const gchar *title,
-                                gboolean verbose);
-RemoteViewer* remote_viewer_new_with_controller(gboolean verbose);
+RemoteViewer* remote_viewer_new(const gchar *uri);
+RemoteViewer* remote_viewer_new_with_controller(void);
 
 G_END_DECLS
 
