@@ -29,6 +29,7 @@
  *    moused-over and auto-closes when the mouse leaves.
  */
 
+#include <config.h>
 
 #include "autoDrawer.h"
 
@@ -214,11 +215,22 @@ ViewAutoDrawerUpdate(ViewAutoDrawer *that, // IN
 
    /* Is the mouse cursor inside the event box? */
 
-   {
+   if (gtk_widget_get_window(priv->evBox)) {
       int x;
       int y;
+#if GTK_CHECK_VERSION(3, 0, 0)
+      GdkDevice *dev;
+      GdkDeviceManager *devmgr;
 
+      devmgr = gdk_display_get_device_manager(gtk_widget_get_display(priv->evBox));
+      dev = gdk_device_manager_get_client_pointer(devmgr);
+
+      gdk_window_get_device_position(gtk_widget_get_window(priv->evBox),
+                                     dev, &x, &y, NULL);
+#else
       gtk_widget_get_pointer(priv->evBox, &x, &y);
+#endif
+
       gtk_widget_get_allocation(priv->evBox, &allocation);
       g_assert(gtk_container_get_border_width(   GTK_CONTAINER(priv->evBox))
                                               == 0);
@@ -257,7 +269,7 @@ ViewAutoDrawerUpdate(ViewAutoDrawer *that, // IN
       }
 #else
       if (window->group && window->group->grabs) {
-	grabbed = GTK_WIDGET(window->group->grabs->data);
+        grabbed = GTK_WIDGET(window->group->grabs->data);
       }
 #endif
       if (!grabbed) {
@@ -440,7 +452,7 @@ ViewAutoDrawerOnSetFocus(GtkWindow *window G_GNUC_UNUSED,    // IN
 
 static void
 ViewAutoDrawerOnHierarchyChanged(ViewAutoDrawer *that,   // IN
-				 GtkWidget *oldToplevel) // IN
+                                 GtkWidget *oldToplevel) // IN
 {
    GtkWidget *newToplevel = gtk_widget_get_toplevel(GTK_WIDGET(that));
 
@@ -653,7 +665,7 @@ ViewAutoDrawerClassInit(gpointer klass) // IN
 
    ovBoxClass->set_over = ViewAutoDrawerSetOver;
 
-   g_type_class_add_private(objectClass, sizeof(ViewAutoDrawerPrivate));
+   g_type_class_add_private(klass, sizeof(ViewAutoDrawerPrivate));
 }
 
 
@@ -689,7 +701,7 @@ ViewAutoDrawer_GetType(void)
          sizeof (ViewAutoDrawer),
          0, /* n_preallocs */
          (GInstanceInitFunc)ViewAutoDrawerInit,
-	 NULL,
+         NULL,
       };
 
       type = g_type_register_static(VIEW_TYPE_DRAWER, "ViewAutoDrawer", &info, 0);
