@@ -112,6 +112,14 @@ virt_viewer_display_vnc_key_ungrab(VncDisplay *vnc G_GNUC_UNUSED,
     g_signal_emit_by_name(display, "display-keyboard-ungrab");
 }
 
+static void
+virt_viewer_display_vnc_initialized(VncDisplay *vnc G_GNUC_UNUSED,
+                                    VirtViewerDisplay *display)
+{
+    virt_viewer_display_set_show_hint(display,
+                                      VIRT_VIEWER_DISPLAY_SHOW_HINT_READY, TRUE);
+}
+
 static void
 virt_viewer_display_vnc_send_keys(VirtViewerDisplay* display,
                                   const guint *keyvals,
@@ -150,7 +158,7 @@ virt_viewer_display_vnc_resize_desktop(VncDisplay *vnc G_GNUC_UNUSED,
                                        int width, int height,
                                        VirtViewerDisplay *display)
 {
-    DEBUG_LOG("desktop resize %dx%d", width, height);
+    g_debug("desktop resize %dx%d", width, height);
 
     virt_viewer_display_set_desktop_size(display, width, height);
 }
@@ -194,6 +202,8 @@ virt_viewer_display_vnc_new(VncDisplay *vnc)
                      G_CALLBACK(virt_viewer_display_vnc_key_grab), display);
     g_signal_connect(display->priv->vnc, "vnc-keyboard-ungrab",
                      G_CALLBACK(virt_viewer_display_vnc_key_ungrab), display);
+    g_signal_connect(display->priv->vnc, "vnc-initialized",
+                     G_CALLBACK(virt_viewer_display_vnc_initialized), display);
 
     return GTK_WIDGET(display);
 }
