@@ -1,7 +1,7 @@
 /*
  * Virt Viewer: A virtual machine console viewer
  *
- * Copyright (C) 2007-2012 Red Hat, Inc.
+ * Copyright (C) 2007-2012, 2014 Red Hat, Inc.
  * Copyright (C) 2009-2012 Daniel P. Berrange
  * Copyright (C) 2010 Marc-André Lureau
  *
@@ -29,6 +29,8 @@
 
 #include <spice-option.h>
 #include <spice-util.h>
+#include <spice-client.h>
+
 #include <usb-device-widget.h>
 #include "virt-viewer-file.h"
 #include "virt-viewer-util.h"
@@ -42,6 +44,10 @@
 #include "gbinding.c"
 #endif
 
+#ifndef SPICE_GTK_CHECK_VERSION
+#define SPICE_GTK_CHECK_VERSION(x, y, z) 0
+#endif
+
 G_DEFINE_TYPE (VirtViewerSessionSpice, virt_viewer_session_spice, VIRT_VIEWER_TYPE_SESSION)
 
 
@@ -409,7 +415,7 @@ fill_session(VirtViewerFile *file, SpiceSession *session)
     }
 
     if (virt_viewer_file_is_set(file, "disable-channels")) {
-        DEBUG_LOG("FIXME: disable-channels is not supported atm");
+        g_debug("FIXME: disable-channels is not supported atm");
     }
 }
 
@@ -472,37 +478,38 @@ virt_viewer_session_spice_main_channel_event(SpiceChannel *channel G_GNUC_UNUSED
                                              VirtViewerSession *session)
 {
     VirtViewerSessionSpice *self = VIRT_VIEWER_SESSION_SPICE(session);
-    gchar *password = NULL;
+    gchar *password = NULL, *user = NULL;
+    int ret;
 
     g_return_if_fail(self != NULL);
 
     switch (event) {
     case SPICE_CHANNEL_OPENED:
-        DEBUG_LOG("main channel: opened");
+        g_debug("main channel: opened");
         g_signal_emit_by_name(session, "session-connected");
         break;
     case SPICE_CHANNEL_CLOSED:
-        DEBUG_LOG("main channel: closed");
+        g_debug("main channel: closed");
         /* Ensure the other channels get closed too */
         virt_viewer_session_clear_displays(session);
         if (self->priv->session)
             spice_session_disconnect(self->priv->session);
         break;
     case SPICE_CHANNEL_SWITCHING:
-        DEBUG_LOG("main channel: switching host");
+        g_debug("main channel: switching host");
         break;
     case SPICE_CHANNEL_ERROR_AUTH:
-        DEBUG_LOG("main channel: auth failure (wrong password?)");
+        g_debug("main channel: auth failure (wrong password?)");
 
         if (self->priv->pass_try > 0)
             g_signal_emit_by_name(session, "session-auth-failed",
                                   _("invalid password"));
         self->priv->pass_try++;
 
-        int ret = virt_viewer_auth_collect_credentials(self->priv->main_window,
-                                                       "SPICE",
-                                                       NULL,
-                                                       NULL, &password);
+        ret = virt_viewer_auth_collect_credentials(self->priv->main_window,
+                                                   "SPICE",
+                                                   NULL,
+                                                   NULL, &password);
         if (ret < 0) {
             g_signal_emit_by_name(session, "session-cancelled");
         } else {
@@ -518,13 +525,40 @@ virt_viewer_session_spice_main_channel_event(SpiceChannel *channel G_GNUC_UNUSED
         }
         break;
     case SPICE_CHANNEL_ERROR_CONNECT:
-        DEBUG_LOG("main channel: failed to connect");
-        g_signal_emit_by_name(session, "session-disconnected");
+#if SPICE_GTK_CHECK_VERSION(0, 23, 21)
+    {
+        const GError *error = spice_channel_get_error(channel);
+
+        g_debug("main channel: failed to connect %s", error ? error->message : "");
+
+        if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_PROXY_NEED_AUTH) ||
+            g_error_matches(error, G_IO_ERROR, G_IO_ERROR_PROXY_AUTH_FAILED)) {
+            SpiceURI *proxy = spice_session_get_proxy_uri(self->priv->session);
+            g_warn_if_fail(proxy != NULL);
+
+            ret = virt_viewer_auth_collect_credentials(self->priv->main_window,
+                                                       "proxy", NULL,
+                                                       &user, &password);
+            if (ret < 0) {
+                g_signal_emit_by_name(session, "session-cancelled");
+            } else {
+                spice_uri_set_user(proxy, user);
+                spice_uri_set_password(proxy, password);
+                spice_session_connect(self->priv->session);
+            }
+        } else {
+            g_signal_emit_by_name(session, "session-disconnected", error->message);
+        }
+    }
+#else
+        g_debug("main channel: failed to connect");
+        g_signal_emit_by_name(session, "session-disconnected", NULL);
+#endif
         break;
     case SPICE_CHANNEL_ERROR_IO:
     case SPICE_CHANNEL_ERROR_LINK:
     case SPICE_CHANNEL_ERROR_TLS:
-        g_signal_emit_by_name(session, "session-disconnected");
+        g_signal_emit_by_name(session, "session-disconnected", NULL);
         break;
     default:
         g_warning("unhandled spice main channel event: %d", event);
@@ -532,6 +566,7 @@ virt_viewer_session_spice_main_channel_event(SpiceChannel *channel G_GNUC_UNUSED
     }
 
     g_free(password);
+    g_free(user);
 }
 
 static void remove_cb(GtkContainer   *container G_GNUC_UNUSED,
@@ -591,7 +626,7 @@ destroy_display(gpointer data)
     VirtViewerDisplay *display = VIRT_VIEWER_DISPLAY(data);
     VirtViewerSession *session = virt_viewer_display_get_session(display);
 
-    DEBUG_LOG("Destroying spice display %p", display);
+    g_debug("Destroying spice display %p", display);
     virt_viewer_session_remove_display(session, display);
     g_object_unref(display);
 }
@@ -627,8 +662,8 @@ virt_viewer_session_spice_display_monitors(SpiceChannel *channel,
         display = g_ptr_array_index(displays, i);
         if (display == NULL) {
             display = virt_viewer_display_spice_new(self, channel, i);
-            DEBUG_LOG("creating spice display (#:%d)", i);
-            g_ptr_array_index(displays, i) = g_object_ref(display);
+            g_debug("creating spice display (#:%d)", i);
+            g_ptr_array_index(displays, i) = g_object_ref_sink(display);
         }
 
         virt_viewer_session_add_display(VIRT_VIEWER_SESSION(self),
@@ -667,7 +702,7 @@ virt_viewer_session_spice_channel_new(SpiceSession *s,
 
     g_object_get(channel, "channel-id", &id, NULL);
 
-    DEBUG_LOG("New spice channel %p %s %d", channel, g_type_name(G_OBJECT_TYPE(channel)), id);
+    g_debug("New spice channel %p %s %d", channel, g_type_name(G_OBJECT_TYPE(channel)), id);
 
     if (SPICE_IS_MAIN_CHANNEL(channel)) {
         if (self->priv->main_channel != NULL)
@@ -695,17 +730,17 @@ virt_viewer_session_spice_channel_new(SpiceSession *s,
     }
 
     if (SPICE_IS_INPUTS_CHANNEL(channel)) {
-        DEBUG_LOG("new inputs channel");
+        g_debug("new inputs channel");
     }
 
     if (SPICE_IS_PLAYBACK_CHANNEL(channel)) {
-        DEBUG_LOG("new audio channel");
+        g_debug("new audio channel");
         if (self->priv->audio == NULL)
             self->priv->audio = spice_audio_get(s, NULL);
     }
 
     if (SPICE_IS_USBREDIR_CHANNEL(channel)) {
-        DEBUG_LOG("new usbredir channel");
+        g_debug("new usbredir channel");
         self->priv->usbredir_channel_count++;
         if (spice_usb_device_manager_get(self->priv->session, NULL))
             virt_viewer_session_set_has_usbredir(session, TRUE);
@@ -736,7 +771,7 @@ virt_viewer_session_spice_fullscreen_auto_conf(VirtViewerSessionSpice *self)
     /* only do auto-conf once at startup. Avoid repeating auto-conf later due to
      * agent disconnection/re-connection, etc */
     if (self->priv->did_auto_conf) {
-        DEBUG_LOG("Already did auto-conf, skipping");
+        g_debug("Already did auto-conf, skipping");
         return FALSE;
     }
 
@@ -744,16 +779,16 @@ virt_viewer_session_spice_fullscreen_auto_conf(VirtViewerSessionSpice *self)
     g_return_val_if_fail(VIRT_VIEWER_IS_APP(app), TRUE);
 
     if (!virt_viewer_app_get_fullscreen(app)) {
-        DEBUG_LOG("app is not in full screen");
+        g_debug("app is not in full screen");
         return FALSE;
     }
     if (cmain == NULL) {
-        DEBUG_LOG("no main channel yet");
+        g_debug("no main channel yet");
         return FALSE;
     }
     g_object_get(cmain, "agent-connected", &agent_connected, NULL);
     if (!agent_connected) {
-        DEBUG_LOG("Agent not connected, skipping autoconf");
+        g_debug("Agent not connected, skipping autoconf");
         g_signal_connect(cmain, "notify::agent-connected", G_CALLBACK(property_notify_do_auto_conf), self);
         return FALSE;
     }
@@ -761,12 +796,12 @@ virt_viewer_session_spice_fullscreen_auto_conf(VirtViewerSessionSpice *self)
     spice_main_set_display_enabled(cmain, -1, FALSE);
 
     ndisplays = virt_viewer_app_get_n_initial_displays(app);
-    DEBUG_LOG("Performing full screen auto-conf, %zd host monitors", ndisplays);
+    g_debug("Performing full screen auto-conf, %" G_GSIZE_FORMAT " host monitors", ndisplays);
 
     for (i = 0; i < ndisplays; i++) {
         gint j = virt_viewer_app_get_initial_monitor_for_display(app, i);
         gdk_screen_get_monitor_geometry(screen, j, &dest);
-        DEBUG_LOG("Set SPICE display %d to (%d,%d)-(%dx%d)",
+        g_debug("Set SPICE display %d to (%d,%d)-(%dx%d)",
                   i, dest.x, dest.y, dest.width, dest.height);
         spice_main_set_display(cmain, i, dest.x, dest.y, dest.width, dest.height);
         spice_main_set_display_enabled(cmain, i, TRUE);
@@ -788,26 +823,26 @@ virt_viewer_session_spice_channel_destroy(G_GNUC_UNUSED SpiceSession *s,
     g_return_if_fail(self != NULL);
 
     g_object_get(channel, "channel-id", &id, NULL);
-    DEBUG_LOG("Destroy SPICE channel %s %d", g_type_name(G_OBJECT_TYPE(channel)), id);
+    g_debug("Destroy SPICE channel %s %d", g_type_name(G_OBJECT_TYPE(channel)), id);
 
     if (SPICE_IS_MAIN_CHANNEL(channel)) {
-        DEBUG_LOG("zap main channel");
+        g_debug("zap main channel");
         if (channel == SPICE_CHANNEL(self->priv->main_channel))
             self->priv->main_channel = NULL;
     }
 
     if (SPICE_IS_DISPLAY_CHANNEL(channel)) {
-        DEBUG_LOG("zap display channel (#%d)", id);
+        g_debug("zap display channel (#%d)", id);
         g_object_set_data(G_OBJECT(channel), "virt-viewer-displays", NULL);
     }
 
     if (SPICE_IS_PLAYBACK_CHANNEL(channel) && self->priv->audio) {
-        DEBUG_LOG("zap audio channel");
+        g_debug("zap audio channel");
         self->priv->audio = NULL;
     }
 
     if (SPICE_IS_USBREDIR_CHANNEL(channel)) {
-        DEBUG_LOG("zap usbredir channel");
+        g_debug("zap usbredir channel");
         self->priv->usbredir_channel_count--;
         if (self->priv->usbredir_channel_count == 0)
             virt_viewer_session_set_has_usbredir(session, FALSE);
@@ -815,7 +850,7 @@ virt_viewer_session_spice_channel_destroy(G_GNUC_UNUSED SpiceSession *s,
 
     self->priv->channel_count--;
     if (self->priv->channel_count == 0)
-        g_signal_emit_by_name(self, "session-disconnected");
+        g_signal_emit_by_name(self, "session-disconnected", NULL);
 }
 
 #define UUID_LEN 16
