@@ -25,6 +25,7 @@
 #include <config.h>
 
 #include <locale.h>
+#include <math.h>
 
 #include "virt-viewer-session.h"
 #include "virt-viewer-util.h"
@@ -37,6 +38,7 @@ struct _VirtViewerSessionPrivate
     GList *displays;
     VirtViewerApp *app;
     gboolean auto_usbredir;
+    gboolean has_usbredir;
     gchar *uri;
     VirtViewerFile *file;
 };
@@ -48,7 +50,9 @@ enum {
 
     PROP_APP,
     PROP_AUTO_USBREDIR,
-    PROP_FILE
+    PROP_HAS_USBREDIR,
+    PROP_FILE,
+    PROP_SW_SMARTCARD_READER,
 };
 
 static void
@@ -82,6 +86,10 @@ virt_viewer_session_set_property(GObject *object,
         virt_viewer_session_set_auto_usbredir(self, g_value_get_boolean(value));
         break;
 
+    case PROP_HAS_USBREDIR:
+        self->priv->has_usbredir = g_value_get_boolean(value);
+        break;
+
     case PROP_APP:
         self->priv->app = g_value_get_object(value);
         break;
@@ -109,6 +117,10 @@ virt_viewer_session_get_property(GObject *object,
         g_value_set_boolean(value, virt_viewer_session_get_auto_usbredir(self));
         break;
 
+    case PROP_HAS_USBREDIR:
+        g_value_set_boolean(value, self->priv->has_usbredir);
+        break;
+
     case PROP_APP:
         g_value_set_object(value, self->priv->app);
         break;
@@ -117,6 +129,10 @@ virt_viewer_session_get_property(GObject *object,
         g_value_set_object(value, self->priv->file);
         break;
 
+    case PROP_SW_SMARTCARD_READER:
+        g_value_set_boolean(value, FALSE);
+        break;
+
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -142,6 +158,16 @@ virt_viewer_session_class_init(VirtViewerSessionClass *class)
                                                          G_PARAM_CONSTRUCT |
                                                          G_PARAM_STATIC_STRINGS));
 
+    g_object_class_install_property(object_class,
+                                    PROP_HAS_USBREDIR,
+                                    g_param_spec_boolean("has-usbredir",
+                                                         "has USB redirection",
+                                                         "has USB redirection",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT |
+                                                         G_PARAM_STATIC_STRINGS));
+
     g_object_class_install_property(object_class,
                                     PROP_APP,
                                     g_param_spec_object("app",
@@ -162,6 +188,15 @@ virt_viewer_session_class_init(VirtViewerSessionClass *class)
                                                          G_PARAM_CONSTRUCT |
                                                          G_PARAM_STATIC_STRINGS));
 
+    g_object_class_install_property(object_class,
+                                    PROP_SW_SMARTCARD_READER,
+                                    g_param_spec_boolean("software-smartcard-reader",
+                                                         "Software smartcard reader",
+                                                         "Indicates whether a software smartcard reader is available",
+                                                         FALSE,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_STATIC_STRINGS));
+
     g_signal_new("session-connected",
                  G_OBJECT_CLASS_TYPE(object_class),
                  G_SIGNAL_RUN_FIRST,
@@ -185,9 +220,10 @@ virt_viewer_session_class_init(VirtViewerSessionClass *class)
                  G_SIGNAL_RUN_FIRST,
                  G_STRUCT_OFFSET(VirtViewerSessionClass, session_disconnected),
                  NULL, NULL,
-                 g_cclosure_marshal_VOID__VOID,
+                 g_cclosure_marshal_VOID__STRING,
                  G_TYPE_NONE,
-                 0);
+                 1,
+                 G_TYPE_STRING);
 
     g_signal_new("session-channel-open",
                  G_OBJECT_CLASS_TYPE(object_class),
@@ -303,12 +339,100 @@ virt_viewer_session_init(VirtViewerSession *session)
     session->priv = VIRT_VIEWER_SESSION_GET_PRIVATE(session);
 }
 
-GtkWidget*
-virt_viewer_session_new(void)
+/* simple sorting of monitors. Primary sort left-to-right, secondary sort from
+ * top-to-bottom, finally by monitor id */
+static int
+displays_cmp(const void *p1, const void *p2, gpointer user_data)
 {
-    return g_object_new(VIRT_VIEWER_TYPE_SESSION, NULL);
+    guint diff;
+    GdkRectangle *displays = user_data;
+    guint i = *(guint*)p1;
+    guint j = *(guint*)p2;
+    GdkRectangle *m1 = &displays[i];
+    GdkRectangle *m2 = &displays[j];
+    diff = m1->x - m2->x;
+    if (diff == 0)
+        diff = m1->y - m2->y;
+    if (diff == 0)
+        diff = i - j;
+
+    return diff;
 }
 
+static void
+virt_viewer_session_align_monitors_linear(GdkRectangle *displays, guint ndisplays)
+{
+    gint i, x = 0;
+    guint *sorted_displays;
+
+    g_return_if_fail(displays != NULL);
+
+    if (ndisplays == 0)
+        return;
+
+    sorted_displays = g_new0(guint, ndisplays);
+    for (i = 0; i < ndisplays; i++)
+        sorted_displays[i] = i;
+    g_qsort_with_data(sorted_displays, ndisplays, sizeof(guint), displays_cmp, displays);
+
+    /* adjust monitor positions so that there's no gaps or overlap between
+     * monitors */
+    for (i = 0; i < ndisplays; i++) {
+        guint nth = sorted_displays[i];
+        g_assert(nth < ndisplays);
+        GdkRectangle *rect = &displays[nth];
+        rect->x = x;
+        rect->y = 0;
+        x += rect->width;
+    }
+    g_free(sorted_displays);
+}
+
+static void
+virt_viewer_session_on_monitor_geometry_changed(VirtViewerSession* self,
+                                                VirtViewerDisplay* display G_GNUC_UNUSED)
+{
+    VirtViewerSessionClass *klass;
+    gboolean all_fullscreen = TRUE;
+    guint nmonitors = 0;
+    GdkRectangle *monitors = NULL;
+    GList *l;
+
+    klass = VIRT_VIEWER_SESSION_GET_CLASS(self);
+    if (!klass->apply_monitor_geometry)
+        return;
+
+    /* find highest monitor ID so we can create the sparse array */
+    for (l = self->priv->displays; l; l = l->next) {
+        VirtViewerDisplay *d = VIRT_VIEWER_DISPLAY(l->data);
+        guint nth = 0;
+        g_object_get(d, "nth-display", &nth, NULL);
+
+        nmonitors = MAX(nth + 1, nmonitors);
+    }
+
+    monitors = g_new0(GdkRectangle, nmonitors);
+    for (l = self->priv->displays; l; l = l->next) {
+        VirtViewerDisplay *d = VIRT_VIEWER_DISPLAY(l->data);
+        guint nth = 0;
+        GdkRectangle *rect = NULL;
+
+        g_object_get(d, "nth-display", &nth, NULL);
+        g_return_if_fail(nth < nmonitors);
+        rect = &monitors[nth];
+        virt_viewer_display_get_preferred_monitor_geometry(d, rect);
+
+        if (virt_viewer_display_get_enabled(d) &&
+            !virt_viewer_display_get_fullscreen(d))
+            all_fullscreen = FALSE;
+    }
+
+    if (!all_fullscreen)
+        virt_viewer_session_align_monitors_linear(monitors, nmonitors);
+
+    klass->apply_monitor_geometry(self, monitors, nmonitors);
+    g_free(monitors);
+}
 
 void virt_viewer_session_add_display(VirtViewerSession *session,
                                      VirtViewerDisplay *display)
@@ -319,6 +443,10 @@ void virt_viewer_session_add_display(VirtViewerSession *session,
     session->priv->displays = g_list_append(session->priv->displays, display);
     g_object_ref(display);
     g_signal_emit_by_name(session, "session-display-added", display);
+
+    virt_viewer_signal_connect_object(display, "monitor-geometry-changed",
+                                      G_CALLBACK(virt_viewer_session_on_monitor_geometry_changed), session,
+                                      G_CONNECT_SWAPPED);
 }
 
 
@@ -429,17 +557,22 @@ gboolean virt_viewer_session_get_auto_usbredir(VirtViewerSession *self)
     return self->priv->auto_usbredir;
 }
 
-gboolean virt_viewer_session_has_usb(VirtViewerSession *self)
+void virt_viewer_session_set_has_usbredir(VirtViewerSession *self, gboolean has_usbredir)
 {
-    VirtViewerSessionClass *klass;
+    g_return_if_fail(VIRT_VIEWER_IS_SESSION(self));
 
+    if (self->priv->has_usbredir == has_usbredir)
+        return;
+
+    self->priv->has_usbredir = has_usbredir;
+    g_object_notify(G_OBJECT(self), "has-usbredir");
+}
+
+gboolean virt_viewer_session_get_has_usbredir(VirtViewerSession *self)
+{
     g_return_val_if_fail(VIRT_VIEWER_IS_SESSION(self), FALSE);
 
-    klass = VIRT_VIEWER_SESSION_GET_CLASS(self);
-    if (klass->has_usb == NULL)
-        return FALSE;
-
-    return klass->has_usb(self);
+    return self->priv->has_usbredir;
 }
 
 void virt_viewer_session_usb_device_selection(VirtViewerSession   *self,
