@@ -65,8 +65,9 @@ G_DEFINE_TYPE (VirtViewer, virt_viewer, VIRT_VIEWER_TYPE_APP)
 
 static gboolean virt_viewer_initial_connect(VirtViewerApp *self, GError **error);
 static gboolean virt_viewer_open_connection(VirtViewerApp *self, int *fd);
-static void virt_viewer_deactivated(VirtViewerApp *self);
+static void virt_viewer_deactivated(VirtViewerApp *self, gboolean connect_error);
 static gboolean virt_viewer_start(VirtViewerApp *self);
+static void virt_viewer_dispose (GObject *object);
 
 static void
 virt_viewer_get_property (GObject *object, guint property_id,
@@ -88,18 +89,6 @@ virt_viewer_set_property (GObject *object, guint property_id,
     }
 }
 
-static void
-virt_viewer_dispose (GObject *object)
-{
-    VirtViewer *self = VIRT_VIEWER(object);
-    VirtViewerPrivate *priv = self->priv;
-    if (priv->dom)
-        virDomainFree(priv->dom);
-    if (priv->conn)
-        virConnectClose(priv->conn);
-    G_OBJECT_CLASS(virt_viewer_parent_class)->dispose (object);
-}
-
 static void
 virt_viewer_class_init (VirtViewerClass *klass)
 {
@@ -125,7 +114,7 @@ virt_viewer_init(VirtViewer *self)
 }
 
 static void
-virt_viewer_deactivated(VirtViewerApp *app)
+virt_viewer_deactivated(VirtViewerApp *app, gboolean connect_error)
 {
     VirtViewer *self = VIRT_VIEWER(app);
     VirtViewerPrivate *priv = self->priv;
@@ -137,14 +126,14 @@ virt_viewer_deactivated(VirtViewerApp *app)
 
     if (priv->reconnect) {
         if (!priv->withEvents) {
-            DEBUG_LOG("No domain events, falling back to polling");
+            g_debug("No domain events, falling back to polling");
             virt_viewer_app_start_reconnect_poll(app);
         }
 
         virt_viewer_app_show_status(app, _("Waiting for guest domain to re-start"));
         virt_viewer_app_trace(app, "Guest %s display has disconnected, waiting to reconnect", priv->domkey);
     } else {
-        VIRT_VIEWER_APP_CLASS(virt_viewer_parent_class)->deactivated(app);
+        VIRT_VIEWER_APP_CLASS(virt_viewer_parent_class)->deactivated(app, connect_error);
     }
 }
 
@@ -335,30 +324,30 @@ virt_viewer_extract_connect_info(VirtViewer *self,
         goto cleanup;
 
     xpath = g_strdup_printf("string(/domain/devices/graphics[@type='%s']/@port)", type);
-    if ((gport = virt_viewer_extract_xpath_string(xmldesc, xpath)) == NULL) {
-        free(xpath);
+    gport = virt_viewer_extract_xpath_string(xmldesc, xpath);
+    g_free(xpath);
+    if (g_str_equal(type, "spice")) {
+        xpath = g_strdup_printf("string(/domain/devices/graphics[@type='%s']/@tlsPort)", type);
+        gtlsport = virt_viewer_extract_xpath_string(xmldesc, xpath);
+        g_free(xpath);
+    }
+
+    if (gport || gtlsport) {
+        xpath = g_strdup_printf("string(/domain/devices/graphics[@type='%s']/@listen)", type);
+        ghost = virt_viewer_extract_xpath_string(xmldesc, xpath);
+    } else {
         xpath = g_strdup_printf("string(/domain/devices/graphics[@type='%s']/@socket)", type);
         if ((unixsock = virt_viewer_extract_xpath_string(xmldesc, xpath)) == NULL) {
             virt_viewer_app_simple_message_dialog(app, _("Cannot determine the graphic address for the guest %s"),
                                                   priv->domkey);
             goto cleanup;
         }
-    } else {
-        if (g_str_equal(type, "spice")) {
-            free(xpath);
-            xpath = g_strdup_printf("string(/domain/devices/graphics[@type='%s']/@tlsPort)", type);
-            gtlsport = virt_viewer_extract_xpath_string(xmldesc, xpath);
-        }
-
-        free(xpath);
-        xpath = g_strdup_printf("string(/domain/devices/graphics[@type='%s']/@listen)", type);
-        ghost = virt_viewer_extract_xpath_string(xmldesc, xpath);
     }
 
     if (ghost && gport)
-        DEBUG_LOG("Guest graphics address is %s:%s", ghost, gport);
+        g_debug("Guest graphics address is %s:%s", ghost, gport);
     else if (unixsock)
-        DEBUG_LOG("Guest graphics address is %s", unixsock);
+        g_debug("Guest graphics address is %s", unixsock);
 
     uri = virConnectGetURI(priv->conn);
     if (virt_viewer_util_extract_host(uri, NULL, &host, &transport, &user, &port) < 0) {
@@ -376,12 +365,13 @@ virt_viewer_extract_connect_info(VirtViewer *self,
      */
     if (virt_viewer_replace_host(ghost)) {
         gchar *replacement_host = NULL;
-        if (g_strcmp0(transport, "ssh") == 0) {
+        if ((g_strcmp0(transport, "ssh") == 0)
+                && !virt_viewer_app_get_direct(app)) {
             replacement_host = g_strdup("localhost");
         } else {
             replacement_host = g_strdup(host);
         }
-        DEBUG_LOG("Guest graphics listen '%s' is NULL or a wildcard, replacing with '%s'",
+        g_debug("Guest graphics listen '%s' is NULL or a wildcard, replacing with '%s'",
                   ghost ? ghost : "", replacement_host);
         g_free(ghost);
         ghost = replacement_host;
@@ -420,7 +410,7 @@ virt_viewer_update_display(VirtViewer *self, virDomainPtr dom)
     virt_viewer_app_trace(app, "Guest %s is running, determining display",
                           priv->domkey);
 
-    g_object_set(app, "title", virDomainGetName(dom), NULL);
+    g_object_set(app, "guest-name", virDomainGetName(dom), NULL);
 
     if (!virt_viewer_app_has_session(app)) {
         if (!virt_viewer_extract_connect_info(self, dom))
@@ -449,7 +439,7 @@ virt_viewer_open_connection(VirtViewerApp *self G_GNUC_UNUSED, int *fd)
     if (virDomainOpenGraphics(priv->dom, 0, pair[0],
                               VIR_DOMAIN_OPEN_GRAPHICS_SKIPAUTH) < 0) {
         virErrorPtr err = virGetLastError();
-        DEBUG_LOG("Error %s", err && err->message ? err->message : "Unknown");
+        g_debug("Error %s", err && err->message ? err->message : "Unknown");
         close(pair[0]);
         close(pair[1]);
         return TRUE;
@@ -471,7 +461,7 @@ virt_viewer_domain_event(virConnectPtr conn G_GNUC_UNUSED,
     VirtViewerApp *app = VIRT_VIEWER_APP(self);
     GError *error = NULL;
 
-    DEBUG_LOG("Got domain event %d %d", event, detail);
+    g_debug("Got domain event %d %d", event, detail);
 
     if (!virt_viewer_matches_domain(self, dom))
         return 0;
@@ -505,7 +495,7 @@ virt_viewer_conn_event(virConnectPtr conn G_GNUC_UNUSED,
     VirtViewerApp *app = VIRT_VIEWER_APP(self);
     VirtViewerPrivate *priv = self->priv;
 
-    DEBUG_LOG("Got connection event %d", reason);
+    g_debug("Got connection event %d", reason);
 
     virConnectClose(priv->conn);
     priv->conn = NULL;
@@ -513,6 +503,24 @@ virt_viewer_conn_event(virConnectPtr conn G_GNUC_UNUSED,
     virt_viewer_app_start_reconnect_poll(app);
 }
 
+static void
+virt_viewer_dispose (GObject *object)
+{
+    VirtViewer *self = VIRT_VIEWER(object);
+    VirtViewerPrivate *priv = self->priv;
+
+    if (priv->withEvents)
+        virConnectDomainEventDeregister(priv->conn,
+                                        virt_viewer_domain_event);
+    virConnectUnregisterCloseCallback(priv->conn,
+                                      virt_viewer_conn_event);
+    if (priv->dom)
+        virDomainFree(priv->dom);
+    if (priv->conn)
+        virConnectClose(priv->conn);
+    G_OBJECT_CLASS(virt_viewer_parent_class)->dispose (object);
+}
+
 static int virt_viewer_connect(VirtViewerApp *app);
 
 static gboolean
@@ -523,8 +531,9 @@ virt_viewer_initial_connect(VirtViewerApp *app, GError **error)
     gboolean ret = FALSE;
     VirtViewer *self = VIRT_VIEWER(app);
     VirtViewerPrivate *priv = self->priv;
+    char uuid_string[VIR_UUID_STRING_BUFLEN];
 
-    DEBUG_LOG("initial connect");
+    g_debug("initial connect");
 
     if (!priv->conn &&
         virt_viewer_connect(app) < 0) {
@@ -543,14 +552,20 @@ virt_viewer_initial_connect(VirtViewerApp *app, GError **error)
         } else {
             virt_viewer_app_simple_message_dialog(app, _("Cannot find guest domain %s"),
                                                   priv->domkey);
-            DEBUG_LOG("Cannot find guest %s", priv->domkey);
+            g_debug("Cannot find guest %s", priv->domkey);
             goto cleanup;
         }
     }
 
+    if (virDomainGetUUIDString(dom, uuid_string) < 0) {
+        g_debug("Couldn't get uuid from libvirt");
+    } else {
+        virt_viewer_app_set_uuid_string(app, uuid_string);
+    }
+
     virt_viewer_app_show_status(app, _("Checking guest domain status"));
     if (virDomainGetInfo(dom, &info) < 0) {
-        DEBUG_LOG("Cannot get guest state");
+        g_debug("Cannot get guest state");
         goto cleanup;
     }
 
@@ -566,7 +581,7 @@ virt_viewer_initial_connect(VirtViewerApp *app, GError **error)
                 virt_viewer_app_trace(app, "Guest %s has not activated its display yet, waiting for it to start",
                                       priv->domkey);
             } else {
-                DEBUG_LOG("Failed to activate viewer");
+                g_debug("Failed to activate viewer");
                 goto cleanup;
             }
         }
@@ -599,7 +614,7 @@ virt_viewer_auth_libvirt_credentials(virConnectCredentialPtr cred,
     int i;
     int ret = -1;
 
-    DEBUG_LOG("Got libvirt credential request for %d credential(s)", ncred);
+    g_debug("Got libvirt credential request for %d credential(s)", ncred);
 
     for (i = 0 ; i < ncred ; i++) {
         switch (cred[i].type) {
@@ -611,7 +626,7 @@ virt_viewer_auth_libvirt_credentials(virConnectCredentialPtr cred,
             password = &cred[i].result;
             break;
         default:
-            DEBUG_LOG("Unsupported libvirt credential %d", cred[i].type);
+            g_debug("Unsupported libvirt credential %d", cred[i].type);
             return -1;
         }
     }
@@ -638,13 +653,13 @@ virt_viewer_auth_libvirt_credentials(virConnectCredentialPtr cred,
                 cred[i].resultlen = strlen(cred[i].result);
             else
                 cred[i].resultlen = 0;
-            DEBUG_LOG("Got '%s' %d %d", cred[i].result, cred[i].resultlen, cred[i].type);
+            g_debug("Got '%s' %d %d", cred[i].result, cred[i].resultlen, cred[i].type);
             break;
         }
     }
 
  cleanup:
-    DEBUG_LOG("Return %d", ret);
+    g_debug("Return %d", ret);
     return ret;
 }
 
@@ -657,7 +672,7 @@ virt_viewer_connect(VirtViewerApp *app)
         { VIR_CRED_AUTHNAME, VIR_CRED_PASSPHRASE };
     virConnectAuth auth_libvirt = {
         .credtype = cred_types,
-        .ncredtype = ARRAY_CARDINALITY(cred_types),
+        .ncredtype = G_N_ELEMENTS(cred_types),
         .cb = virt_viewer_auth_libvirt_credentials,
         .cbdata = app,
     };
@@ -667,7 +682,7 @@ virt_viewer_connect(VirtViewerApp *app)
     if (!virt_viewer_app_get_attach(app))
         oflags |= VIR_CONNECT_RO;
 
-    DEBUG_LOG("connecting ...");
+    g_debug("connecting ...");
 
     virt_viewer_app_trace(app, "Opening connection to libvirt with URI %s",
                           priv->uri ? priv->uri : "<null>");
@@ -683,7 +698,7 @@ virt_viewer_connect(VirtViewerApp *app)
 
     if (!virt_viewer_app_initial_connect(app, &error)) {
         if (error)
-            g_warning(error->message);
+            g_warning("%s", error->message);
         g_clear_error(&error);
         return -1;
     }
@@ -698,7 +713,7 @@ virt_viewer_connect(VirtViewerApp *app)
 
     if (!priv->withEvents &&
         !virt_viewer_app_is_active(app)) {
-        DEBUG_LOG("No domain events, falling back to polling");
+        g_debug("No domain events, falling back to polling");
         virt_viewer_app_start_reconnect_poll(app);
     }
 
@@ -706,7 +721,7 @@ virt_viewer_connect(VirtViewerApp *app)
                                         virt_viewer_conn_event,
                                         self,
                                         NULL) < 0) {
-        DEBUG_LOG("Unable to register close callback on libvirt connection");
+        g_debug("Unable to register close callback on libvirt connection");
     }
 
     return 0;
@@ -728,31 +743,21 @@ virt_viewer_start(VirtViewerApp *app)
 VirtViewer *
 virt_viewer_new(const char *uri,
                 const char *name,
-                gint zoom,
                 gboolean direct,
                 gboolean attach,
                 gboolean waitvm,
-                gboolean reconnect,
-                gboolean verbose,
-                GtkWidget *container)
+                gboolean reconnect)
 {
     VirtViewer *self;
     VirtViewerApp *app;
     VirtViewerPrivate *priv;
 
     self = g_object_new(VIRT_VIEWER_TYPE,
-                        "container", container,
-                        "verbose", verbose,
                         "guest-name", name,
                         NULL);
     app = VIRT_VIEWER_APP(self);
     priv = self->priv;
 
-    /* Set initial title based on guest name arg, which can be a ID,
-     * UUID, or NAME string. To be replaced with the real guest name later
-     */
-    g_object_set(app, "title", name, NULL);
-    virt_viewer_window_set_zoom_level(virt_viewer_app_get_main_window(app), zoom);
     virt_viewer_app_set_direct(app, direct);
     virt_viewer_app_set_attach(app, attach);
 
